﻿using PrecastConcretePlantBusinessLogic.OfficePackage.HelperModels;
using PrecastConcretePlantBusinessLogic.OfficePackage;
using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.BusinessLogicsContracts;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.StoragesContracts;
using PrecastConcretePlantContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PrecastConcretePlantBusinessLogic.BusinessLogics
{
    public class ReportLogic : IReportLogic
    {
        private readonly IComponentStorage _componentStorage;
        private readonly IReinforcedStorage _reinforcedStorage;
        private readonly IOrderStorage _orderStorage;
        private readonly IShopStorage _shopStorage;

        private readonly AbstractSaveToExcel _saveToExcel;
        private readonly AbstractSaveToWord _saveToWord;
        private readonly AbstractSaveToPdf _saveToPdf;
        public ReportLogic(IReinforcedStorage reinforcedStorage, IComponentStorage componentStorage, IOrderStorage orderStorage, IShopStorage shopStorage, AbstractSaveToExcel saveToExcel, AbstractSaveToWord saveToWord, AbstractSaveToPdf saveToPdf)
        {
            _reinforcedStorage = reinforcedStorage;
            _componentStorage = componentStorage;
            _orderStorage = orderStorage;
            _saveToExcel = saveToExcel;
            _saveToWord = saveToWord;
            _saveToPdf = saveToPdf;
            _shopStorage = shopStorage;
        }

        public List<ReportReinforcedComponentViewModel> GetReinforcedComponents()
        {
            var components = _componentStorage.GetFullList();
            var reinforceds = _reinforcedStorage.GetFullList();
            var list = new List<ReportReinforcedComponentViewModel>();
            foreach (var reinforced in reinforceds)
            {
                var record = new ReportReinforcedComponentViewModel
                {
                    ReinforcedName = reinforced.ReinforcedName,
                    Components = new List<(string Component, int Count)>(),
                    TotalCount = 0
                };
                foreach (var component in components)
                {
                    if (reinforced.ReinforcedComponents.ContainsKey(component.Id))
                    {
                        record.Components.Add(new(component.ComponentName, reinforced.ReinforcedComponents[component.Id].Item2));
                        record.TotalCount += reinforced.ReinforcedComponents[component.Id].Item2;
                    }
                }
                list.Add(record);
            }
            return list;
        }

        public List<ReportShopReinforcedViewModel> GetShopReinforceds()
        {
            var shops = _shopStorage.GetFullList();

            var list = new List<ReportShopReinforcedViewModel>();

            foreach (var shop in shops)
            {
                var record = new ReportShopReinforcedViewModel
                {
                    ShopName = shop.ShopName,
                    Reinforceds = new List<(string Reinforced, int Count)>(),
                    TotalCount = 0,
                };
                foreach (var reinforced in shop.ShopReinforceds)
                {
                    record.Reinforceds.Add(new(reinforced.Value.Item1.ReinforcedName, reinforced.Value.Item2));
                    record.TotalCount += reinforced.Value.Item2;
                }

                list.Add(record);
            }

            return list;
        }
        public List<ReportOrdersByDateViewModel> GetGroupedByDateOrders()
        {
            return _orderStorage.GetFullList().GroupBy(x => x.DateCreate.Date)
                     .Select(x => new ReportOrdersByDateViewModel
                     {
                         Date = x.Key,
                         Count = x.Count(),
                         Sum = x.Sum(y => y.Sum)
                     })
                     .ToList();

        }

        public List<ReportOrdersViewModel> GetOrders(ReportBindingModel model)
        {
            List<ReportOrdersViewModel> r = _orderStorage.GetFilteredList(new OrderSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            })
            .Select(x => new ReportOrdersViewModel
            {
                Id = x.Id,
                DateCreate = x.DateCreate,
                ReinforcedName = x.ReinforcedName,
                Sum = x.Sum,
                Status = x.Status.ToString(),
            })
           .ToList();
            return r;
        }

        public void SaveShopsToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateShopsTable(new WordInfo
            {
                FileName = model.FileName,
                Title = "Список магазинов",
                Shops = _shopStorage.GetFullList()
            });
        }
        public void SaveShopReinforcedToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateShopReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Загруженность магазинов",
                ShopReinforceds = GetShopReinforceds()
            });
        }
        public void SaveGroupedOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDocWithGroupedOrders(new PdfInfo
            {
                FileName = model.FileName,
                Title = "Заказы по датам",
                GroupedOrders = GetGroupedByDateOrders()
            });
        }

        public void SaveReinforcedsToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateDoc(new WordInfo
            {
                FileName = model.FileName,
                Title = "Список ЖИ",
                Reinforceds = _reinforcedStorage.GetFullList()
            });
        }
        public void SaveReinforcedComponentToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Список ЖИ",
                ReinforcedComponents = GetReinforcedComponents()
            });
        }

        public void SaveOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDoc(new PdfInfo
            {
                FileName = model.FileName,
                Title = "Список заказов",
                DateFrom = model.DateFrom!.Value,
                DateTo = model.DateTo!.Value,
                Orders = GetOrders(model)
            });
        }

    }
}
